%
% SMU_DISPLAY Virtual front panel display for the Source/Measure Unit.
%    SMU_DISPLAY(SMU) creates a GUI that serves as a virtual front panel 
%    display for the SMU source/measure unit.  The display is updated 
%    approximately 10 times per second.  The source value for a chanel can 
%    be changed via the edit below its display.  If the value entered contains 
%    units, the mode of that chanel will change if necessary.  The units can 
%    also include one of the following prefixes: 'm' for milli, 'u' for micro, 
%    'n' for nano, and 'p' for pico.  If the MODE button of a chanel is pressed, 
%    the last measured value becomes the new source value.  The AUTO button 
%    for a chanel toggles firmware autoranging for that chanel.  The RANGE 
%    buttons for a chanel change its selected measurement range.
function smu_display(smu)

smu_name = inputname(1);

% Test to see if another smu_display window is running.  If so, bring it to the front and return.
fig_handles=get(0, 'children');
smu_display_fig=findobj(fig_handles, 'tag', [smu_name, '_display_fig']);
if length(smu_display_fig)~=0,
    figure(smu_display_fig);
    return;
end

set(0, 'units', 'pixels');
screen_rect=get(0, 'screensize');
screen_width=screen_rect(3); screen_height=screen_rect(4);
figure('units', 'pixels',...
    'position', [(screen_width/2-256), (screen_height/2-92), 512, 184],...
    'numbertitle', 'off',...
    'name', ['USB Source/Measure Unit (', smu_name, ')'],...
    'resize', 'off',...
    'tag', [smu_name, '_display_fig']);

uicontrol('style', 'frame',...
    'backgroundcolor', [0, 0, 0],...
    'position', [8, 72, 244, 104]);
uicontrol('style', 'text',...
    'string', 'CH1',...
    'fontname', 'Courier',...
    'fontweight', 'bold',...
    'fontsize', 12,...
    'backgroundcolor', [0, 0, 0],...
    'foregroundcolor', [0, 1, 1],...
    'position', [10, 156, 36, 18],...
    'horizontalalignment', 'left');
uicontrol('style', 'text',...
    'string', 'AUTO',...
    'fontname', 'Courier',...
    'fontweight', 'bold',...
    'fontsize', 12,...
    'backgroundcolor', [0, 0, 0],...
    'foregroundcolor', [0, 1, 1],...
    'position', [202, 156, 48, 18],...
    'horizontalalignment', 'left',...
    'tag', [smu_name, '_gui_ch1_auto']);
uicontrol('style', 'text',...
    'string', ['---.--', char(181), 'A'],...
    'fontname', 'Courier',...
    'fontweight', 'bold',...
    'fontsize', 36,...
    'backgroundcolor', [0, 0, 0],...
    'foregroundcolor', [0, 1, 1],...
    'position', [10, 104, 240, 52],...
    'horizontalalignment', 'left',...
    'tag', [smu_name, '_gui_ch1_meas']);
uicontrol('style', 'text',...
    'string', 'Vsrc=+0.0000V ',...
    'fontname', 'Courier',...
    'fontweight', 'bold',...
    'fontsize', 21,...
    'backgroundcolor', [0, 0, 0],...
    'foregroundcolor', [0, 1, 1],...
    'position', [10, 74, 240, 30],...
    'horizontalalignment', 'left',...
    'tag', [smu_name, '_gui_ch1_src']);
uicontrol('style', 'pushbutton',...
    'string', 'MODE',...
    'position', [8 40 55 24],...
    'callback', ['[smu_value, smu_units]=', smu_name, '.get_meas(1);',...
        smu_name, '.set_source(1, smu_value, smu_units);',...
        'clear smu_value smu_units;']);
uicontrol('style', 'pushbutton',...
    'string', 'AUTO',...
    'position', [71 40 55 24],...
    'callback', [smu_name, '.set_autorange(1, 1-', smu_name, '.get_autorange(1));']);
uicontrol('style', 'pushbutton',...
    'string', '<',...
    'position', [134 40 28 24],...
    'callback', ['smu_fn=', smu_name, '.get_function(1);',...
        'if smu_fn==0;',...
        '    smu_range=', smu_name, '.get_irange(1);',...
        '    if smu_range~=5;',...
        '        ', smu_name, '.set_irange(1, smu_range+1);',...
        '    end;',...
        'elseif smu_fn==1;',...
        '    smu_range=', smu_name, '.get_vrange(1);',...
        '    if smu_range~=2;',...
        '        ', smu_name, '.set_vrange(1, smu_range+1);',...
        '    end;',...
        'end;',...
        'clear smu_fn smu_range;']);
uicontrol('style', 'text',...
    'string', 'RANGE',...
    'backgroundcolor', [0.8, 0.8, 0.8],...
    'position', [170, 35, 46, 24]);
uicontrol('style', 'pushbutton',...
    'string', '>',...
    'position', [224 40 28 24],...
    'callback', ['smu_fn=', smu_name, '.get_function(1);',...
        'if smu_fn==0;',...
        '    smu_range=', smu_name, '.get_irange(1);',...
        '    if smu_range~=0;',...
        '        ', smu_name, '.set_irange(1, smu_range-1);',...
        '    end;',...
        'elseif smu_fn==1;',...
        '    smu_range=', smu_name, '.get_vrange(1);',...
        '    if smu_range~=0;',...
        '        ', smu_name, '.set_vrange(1, smu_range-1);',...
        '    end;',...
        'end;',...
        'clear smu_fn smu_range;']);
uicontrol('style', 'edit',...
    'position', [8, 8, 244, 24],...
    'horizontalalignment', 'left',...
    'callback', ['smu_ch1_src_edit_h=findobj(''tag'', ''', smu_name, '_ch1_src_edit'');',...
        smu_name, '.set_src_str(1, deblank(get(smu_ch1_src_edit_h, ''string'')));',...
        'set(smu_ch1_src_edit_h, ''string'', '''');',...
        'clear smu_ch1_src_edit_h;'],...
    'tag', [smu_name, '_ch1_src_edit']);

uicontrol('style', 'frame',...
    'backgroundcolor', [0, 0, 0],...
    'position', [260, 72, 244, 104]);
uicontrol('style', 'text',...
    'string', 'CH2',...
    'fontname', 'Courier',...
    'fontweight', 'bold',...
    'fontsize', 12,...
    'backgroundcolor', [0, 0, 0],...
    'foregroundcolor', [0, 1, 1],...
    'position', [262, 156, 36, 18],...
    'horizontalalignment', 'left');
uicontrol('style', 'text',...
    'string', 'AUTO',...
    'fontname', 'Courier',...
    'fontweight', 'bold',...
    'fontsize', 12,...
    'backgroundcolor', [0, 0, 0],...
    'foregroundcolor', [0, 1, 1],...
    'position', [454, 156, 48, 18],...
    'horizontalalignment', 'left',...
    'tag', [smu_name, '_gui_ch2_auto']);
uicontrol('style', 'text',...
    'string', ['---.--', char(181), 'A'],...
    'fontname', 'Courier',...
    'fontweight', 'bold',...
    'fontsize', 36,...
    'backgroundcolor', [0, 0, 0],...
    'foregroundcolor', [0, 1, 1],...
    'position', [262, 104, 240, 52],...
    'horizontalalignment', 'left',...
    'tag', [smu_name, '_gui_ch2_meas']);
uicontrol('style', 'text',...
    'string', 'Vsrc=+0.0000V ',...
    'fontname', 'Courier',...
    'fontweight', 'bold',...
    'fontsize', 21,...
    'backgroundcolor', [0, 0, 0],...
    'foregroundcolor', [0, 1, 1],...
    'position', [262, 74, 240, 30],...
    'horizontalalignment', 'left',...
    'tag', [smu_name, '_gui_ch2_src']);
uicontrol('style', 'pushbutton',...
    'string', 'MODE',...
    'position', [260 40 55 24],...
    'callback', ['[smu_value, smu_units]=', smu_name, '.get_meas(2);',...
        smu_name, '.set_source(2, smu_value, smu_units);',...
        'clear smu_value smu_units;']);
uicontrol('style', 'pushbutton',...
    'string', 'AUTO',...
    'position', [323 40 55 24],...
    'callback', [smu_name, '.set_autorange(2, 1-', smu_name, '.get_autorange(2));']);
uicontrol('style', 'pushbutton',...
    'string', '<',...
    'position', [386 40 28 24],...
    'callback', ['smu_fn=', smu_name, '.get_function(2);',...
        'if smu_fn==0;',...
        '    smu_range=', smu_name, '.get_irange(2);',...
        '    if smu_range~=5;',...
        '        ', smu_name, '.set_irange(2, smu_range+1);',...
        '    end;',...
        'elseif smu_fn==1;',...
        '    smu_range=', smu_name, '.get_vrange(2);',...
        '    if smu_range~=2;',...
        '        ', smu_name, '.set_vrange(2, smu_range+1);',...
        '    end;',...
        'end;',...
        'clear smu_fn smu_range;']);
uicontrol('style', 'text',...
    'string', 'RANGE',...
    'backgroundcolor', [0.8, 0.8, 0.8],...
    'position', [422, 35, 46, 24]);
uicontrol('style', 'pushbutton',...
    'string', '>',...
    'position', [476 40 28 24],...
    'callback', ['smu_fn=', smu_name, '.get_function(2);',...
        'if smu_fn==0;',...
        '    smu_range=', smu_name, '.get_irange(2);',...
        '    if smu_range~=0;',...
        '        ', smu_name, '.set_irange(2, smu_range-1);',...
        '    end;',...
        'elseif smu_fn==1;',...
        '    smu_range=', smu_name, '.get_vrange(1);',...
        '    if smu_range~=0;',...
        '        ', smu_name, '.set_vrange(2, smu_range-1);',...
        '    end;',...
        'end;',...
        'clear smu_fn smu_range;']);
uicontrol('style', 'edit',...
    'position', [260, 8, 244, 24],...
    'horizontalalignment', 'left',...
    'callback', ['smu_ch2_src_edit_h=findobj(''tag'', ''', smu_name, '_ch2_src_edit'');',...
        smu_name, '.set_src_str(2, deblank(get(smu_ch2_src_edit_h, ''string'')));',...
        'set(smu_ch2_src_edit_h, ''string'', '''');',...
        'clear smu_ch2_src_edit_h;'],...
    'tag', [smu_name, '_ch2_src_edit']);
timer('TimerFcn', ['smu_update_display(', smu_name, ');'],...
    'StartDelay', 0.1,...
    'StopFcn', ['smu_timer_h=timerfind(''Tag'', ''', smu_name, '_timer'');',...
        'fig_handles=get(0, ''children'');',...
        'smu_display_fig=findobj(fig_handles, ''tag'', ''', smu_name, '_display_fig'');',...
        'if length(smu_display_fig)~=0;',...
        '    start(smu_timer_h);',...
        'else;',...
        '    delete(smu_timer_h);',...
        'end;',...
        'clear smu_timer_h fig_handles smu_display_fig;'],...
    'Tag', [smu_name, '_timer']);
smu_timer_h = timerfind('Tag', [smu_name, '_timer']);
start(smu_timer_h);
